@@ -37,30 +37,35 @@ igeom <- function(u = runif(1), prob,
                   sampleColor     = "red3",
                   populationColor = "grey",
                   showTitle       = TRUE,
-                  respectLayout   = FALSE, ...)
+                  respectLayout   = FALSE, 
+                  restorePar      = TRUE,  # add 23 Nov 2023
+                  ...)
 {
   #############################################################################
 
   if(is.null(dev.list()))  dev.new(width=5, height=6)
   
-  warnVal <- options("warn")          # save current warning setting...
-  oldpar  <- par(no.readonly = TRUE)  # save current par settings
+  #warnVal <- options("warn")          # save current warning setting... (del 22 Nov 2023)
 
   #############################################################################
 
-  options(warn = -1)          # suppress warnings
+  #options(warn = -1)          # suppress warnings -- remove RE CRAN req't (del 22 Nov 2023)
 
   if (!is.null(u) && (min(u) <= 0 || max(u) >= 1))  stop("must have 0 < u < 1")
 
   checkVal(prob, minex = 0, maxex = 1)
   checkQuants(minPlotQuantile, maxPlotQuantile, min = 0, maxex = 1)
 
-  options(warn = 1)                   # set to immediate warnings
+  #options(warn = 1)                   # set to immediate warnings (del 22 Nov 2023)
 
   # Check for deprecated parameters
   for (arg in names(list(...))) {
-    if (arg == "maxPlotTime")
-      warning("'maxPlotTime' has been deprecated as of simEd v2.0.0")
+    if (arg == "maxPlotTime") {
+      # mod 23 Nov 2023
+      #warning("'maxPlotTime' has been deprecated as of simEd v2.0.0")
+      warning("'maxPlotTime' has been deprecated as of simEd v2.0.0",
+              immediate. = TRUE)
+    }
     else stop(paste("Unknown argument '", arg, "'", sep = ""))
   }
 
@@ -71,7 +76,11 @@ igeom <- function(u = runif(1), prob,
   getDistro   <- function(d)  pgeom(d, prob)  #p
   getQuantile <- function(d)  qgeom(d, prob)  #q
 
-  titleStr <- paste("Geometric (p = ", round(prob, 3), ")", sep = "")
+  # using plotmath for [nothing here]; bquote to use .() to evaluate args;
+  #  in bquote, ~ includes space b/w while * appends w/ no space b/w
+  titleStr <- as.expression(bquote(bold(
+                    "Geometric (" ~ "p" ~ "=" ~ .(round(prob, 3)) ~ ")"
+              )))
 
   #############################################################################
 
@@ -90,6 +99,7 @@ igeom <- function(u = runif(1), prob,
     populationColor  = populationColor,
     showTitle        = showTitle,
     respectLayout    = respectLayout,
+    restorePar       = restorePar,    # add 23 Nov 2023
     getDensity       = getDensity,
     getDistro        = getDistro,
     getQuantile      = getQuantile,
@@ -100,11 +110,13 @@ igeom <- function(u = runif(1), prob,
   )
 
   # resetting par and warning settings
-  options(warn = warnVal$warn)
-  if (!all(oldpar$mfrow == par()$mfrow)) {
-    # ?par claims "restoring all of [oldpar] is not wise", so reset only mfrow
-    par(mfrow = oldpar$mfrow)
-  }
+  #options(warn = warnVal$warn)  # remove RE CRAN req't (del 22 Nov 2023)
+
+  ## using on.exit for par RE CRAN suggest (del 22 Nov 2023)
+  #if (!all(oldpar$mfrow == par()$mfrow)) {
+  #  # ?par claims "restoring all of [oldpar] is not wise", so reset only mfrow
+  #  par(mfrow = oldpar$mfrow)
+  #}
 
   if (!is.null(out)) return(out)
 }
